@@ -4,10 +4,9 @@
 __author__ = 'https://github.com/akvilary'
 
 # core classes
-from .sql_query import SqlQuery
+from .sql_query import SqlQuery, SqlCte
 from .sql_section import SqlSection
 from .sql_container import SqlContainer
-from .sql_cte import SqlCte
 
 # supplementary classes
 from .sql_enum import SqlEnum
@@ -19,3 +18,4 @@ from .sql_section_header import SqlSectionHeader
 from .sql_filter import SqlFilter
 from .sql_filters import SqlFilters
 from .sql_placeholder import SqlPlaceholder
+from .sql_wrap import SqlWrap
